# SPDX-FileCopyrightText: 2025 QinHan
# SPDX-License-Identifier: MPL-2.0
"""
Allow running docutranslate mcp server with `python -m docutranslate.mcp`
"""

from docutranslate.mcp.server import main

if __name__ == "__main__":
    main()
